@@ -36,6 +36,26 @@
 
 void _glade_init_gtk_widgets(void);
 
+static void
+custom_noop (GladeXML *xml, GtkWidget *widget,
+	     const char *name, const char *value)
+{
+    ;
+}
+
+static void
+set_visible(GladeXML *xml, GtkWidget *widget,
+	    const gchar *prop_name, const gchar *prop_value)
+{
+    static GQuark visible_id = 0;
+
+    if (visible_id == 0)
+	visible_id = g_quark_from_static_string("Libglade::visible");
+
+    if (BOOL(prop_value))
+	g_object_set_qdata(G_OBJECT(widget), visible_id,GINT_TO_POINTER(TRUE));
+}
+
 static void
 set_tooltip(GladeXML *xml, GtkWidget *widget,
 	    const gchar *prop_name, const gchar *prop_value)
@@ -82,13 +102,6 @@ progress_set_format (GladeXML *xml, GtkWidget *w,
     gtk_progress_set_format_string (GTK_PROGRESS (w), value);
 }
 
-static void
-option_menu_set_history (GladeXML *xml, GtkWidget *w,
-			 const char *name, const char *value)
-{
-    gtk_option_menu_set_history (GTK_OPTION_MENU (w), INT (value));
-}
-
 static void
 text_view_set_text (GladeXML *xml, GtkWidget *w,
 		    const char *name, const char *value)
@@ -234,6 +247,97 @@ toolbar_set_tooltips (GladeXML *xml, GtkWidget *w,
     gtk_toolbar_set_tooltips (GTK_TOOLBAR (w), BOOL (value));
 }
 
+static void
+statusbar_set_has_resize_grip (GladeXML *xml, GtkWidget *w,
+			       const char *name, const char *value)
+{
+    gtk_statusbar_set_has_resize_grip (GTK_STATUSBAR (w), BOOL (value));
+}
+
+static void
+ruler_set_metric (GladeXML *xml, GtkWidget *w,
+		  const char *name, const char *value)
+{
+    gtk_ruler_set_metric (
+	GTK_RULER (w),
+	glade_enum_from_string (GTK_TYPE_METRIC_TYPE, value));
+}
+
+static void
+menu_item_set_label (GladeXML *xml, GtkWidget *w,
+		     const char *name, const char *value)
+{
+    GtkWidget *child = GTK_BIN (w)->child;
+
+    if (!child) {
+	child = gtk_accel_label_new("");
+	gtk_misc_set_alignment(GTK_MISC(child), 0.0, 0.5);
+	gtk_container_add(GTK_CONTAINER(w), child);
+	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(child), w);
+	gtk_widget_show(child);
+    }
+
+    if (GTK_IS_LABEL (child))
+	gtk_label_set_text (GTK_LABEL (child), value);
+}
+
+static void
+menu_item_set_use_underline (GladeXML *xml, GtkWidget *w,
+			     const char *name, const char *value)
+{
+    GtkWidget *child = GTK_BIN (w)->child;
+
+    if (!child) {
+	child = gtk_accel_label_new("");
+	gtk_misc_set_alignment(GTK_MISC(child), 0.0, 0.5);
+	gtk_container_add(GTK_CONTAINER(w), child);
+	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(child), w);
+	gtk_widget_show(child);
+    }
+
+    if (GTK_IS_LABEL (child))
+	gtk_label_set_use_underline (GTK_LABEL (child), BOOL (value));
+}
+
+static void
+menu_item_set_use_stock (GladeXML *xml, GtkWidget *w,
+			 const gchar *name, const gchar *value)
+{
+    GtkWidget *child = GTK_BIN (w)->child;
+
+    if (!child) {
+	child = gtk_accel_label_new("");
+	gtk_misc_set_alignment(GTK_MISC(child), 0.0, 0.5);
+	gtk_container_add(GTK_CONTAINER(w), child);
+	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(child), w);
+	gtk_widget_show(child);
+    }
+
+    if (GTK_IS_LABEL (child) && BOOL(value)) {
+	const gchar *stock_id = gtk_label_get_label(GTK_LABEL(child));
+	GtkStockItem stock_item;
+
+	if (gtk_stock_lookup(stock_id, &stock_item)) {
+	    /* put in the stock image next to the text.  Done before
+             * messing with the label child, so that stock_id doesn't
+             * become invalid. */
+	    if (GTK_IS_IMAGE_MENU_ITEM(w)) {
+		GtkWidget *image =
+		    gtk_image_new_from_stock(stock_id, GTK_ICON_SIZE_MENU);
+
+		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(w), image);
+		gtk_widget_show(image);
+	    }
+
+	    gtk_label_set_text(GTK_LABEL(child), stock_item.label);
+	    gtk_label_set_use_underline(GTK_LABEL(child), TRUE);
+	} else {
+	    g_warning("could not look up stock id '%s'", stock_id);
+	}
+    }
+}
+
+
 static GtkWidget *
 placeholder_create (void)
 {
@@ -364,6 +468,29 @@ build_preview (GladeXML *xml, GType widget_type,
     return preview;
 }
 
+static void
+option_menu_build_children (GladeXML *xml, GtkWidget *parent,
+			    GladeWidgetInfo *info)
+{
+    int i, history = 0;
+
+    glade_standard_build_children (xml, parent, info);
+
+    for (i = 0; i < info->n_properties; i++) {
+	const char *name  = info->properties[i].name;
+	const char *value = info->properties[i].value;
+
+	if (strcmp (name, "history"))
+	    continue;
+
+	history = INT (value);
+	break;
+    }
+
+    /* we have to set the history *after* building the child menu */
+    gtk_option_menu_set_history (GTK_OPTION_MENU (parent), history);
+}
+
 static void
 clist_build_children(GladeXML *self, GtkWidget *parent,
 		     GladeWidgetInfo *info)
@@ -428,7 +555,8 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 	    !strcmp (childinfo->child->class, "button")) {
 	    const char *label = NULL, *stock = NULL, *group_name = NULL;
 	    char *icon = NULL;
-	    gboolean active = FALSE, new_group = FALSE;
+	    gboolean use_stock = FALSE, active = FALSE, new_group = FALSE;
+	    gboolean use_underline = FALSE;
 	    GtkWidget *iconw = NULL;
 	    int j;
 
@@ -438,6 +566,8 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 
 		if (!strcmp (name, "label")) {
 		    label = value;
+		} else if (!strcmp (name, "use_stock")) {
+		    use_stock = TRUE;
 		} else if (!strcmp (name, "icon")) {
 		    g_free (icon);
 		    stock = NULL;
@@ -456,11 +586,31 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 		    /* ignore for now */
 		} else if (!strcmp (name, "tooltip")) {
 		    /* ignore for now */
+		} else if (!strcmp (name, "use_underline")) {
+		    use_underline = BOOL (value);
+		} else if (!strcmp (name, "inconsistent")) {
+		    /* ignore for now */
 		} else {
 		    g_warning ("Unknown GtkToolbar child property: %s", name);
 		}
 	    }
 
+	    /* For stock items, we create the stock icon and get the label
+	       here, partly because GTK+ doesn't have direct support for stock
+	       toggle & radio items. */
+	    if (use_stock) {
+		GtkStockItem item;
+
+		if (gtk_stock_lookup (label, &item)) {
+		    /* Set stock to label, so the icon is created below. */
+		    stock = label;
+		    label = item.label;
+
+		    /* Most stock items have mnemonic accelerators. */
+		    use_underline = TRUE;
+		}
+	    }
+
 	    if (stock) {
 		iconw = gtk_image_new_from_stock (
 		    stock, GTK_TOOLBAR (parent)->icon_size);
@@ -500,6 +650,15 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 		    GTK_TOOLBAR (parent),
 		    label, NULL, NULL, iconw, NULL, NULL);
 	    
+	    /* GTK+ doesn't support use_underline directly, so we have to hack
+	       it. */
+	    if (use_underline) {
+		GList *elem = g_list_last (GTK_TOOLBAR (parent)->children);
+		GtkToolbarChild *toolbar_child = elem->data;
+		gtk_label_set_use_underline (GTK_LABEL (toolbar_child->label),
+					     TRUE);
+	    }
+
 	    glade_xml_set_common_params (xml, child, childinfo->child);
 	} else {
 	    child = glade_xml_build_widget (xml, childinfo->child);
@@ -603,7 +762,7 @@ image_menu_find_internal_child(GladeXML *xml, GtkWidget *parent,
 
 	pl = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (parent));
 	if (!pl) {
-	    pl = placeholder_create ();
+	    pl = gtk_image_new ();
 
 	    gtk_image_menu_item_set_image (
 		GTK_IMAGE_MENU_ITEM (parent), pl);
@@ -716,11 +875,12 @@ combo_find_internal_child(GladeXML *xml, GtkWidget *parent,
 void
 _glade_init_gtk_widgets(void)
 {
+    glade_register_custom_prop (GTK_TYPE_WIDGET, "visible", set_visible);
     glade_register_custom_prop (GTK_TYPE_WIDGET, "tooltip", set_tooltip);
     glade_register_custom_prop (GTK_TYPE_PIXMAP, "build_insensitive", pixmap_set_build_insensitive);
     glade_register_custom_prop (GTK_TYPE_PIXMAP, "filename", pixmap_set_filename);
     glade_register_custom_prop (GTK_TYPE_PROGRESS, "format", progress_set_format);
-    glade_register_custom_prop (GTK_TYPE_OPTION_MENU, "history", option_menu_set_history);
+    glade_register_custom_prop (GTK_TYPE_OPTION_MENU, "history", custom_noop);
     glade_register_custom_prop (GTK_TYPE_TEXT_VIEW, "text", text_view_set_text);
     glade_register_custom_prop (GTK_TYPE_CALENDAR, "display_options", calendar_set_display_options);
     glade_register_custom_prop (GTK_TYPE_CLIST, "column_widths", clist_set_column_widths);
@@ -737,6 +897,11 @@ _glade_init_gtk_widgets(void)
     glade_register_custom_prop (GTK_TYPE_RADIO_MENU_ITEM, "group",
 				radio_menu_item_set_group);
     glade_register_custom_prop (GTK_TYPE_TOOLBAR, "tooltips", toolbar_set_tooltips);
+    glade_register_custom_prop (GTK_TYPE_STATUSBAR, "has_resize_grip", statusbar_set_has_resize_grip);
+    glade_register_custom_prop (GTK_TYPE_RULER, "metric", ruler_set_metric);
+    glade_register_custom_prop (GTK_TYPE_MENU_ITEM, "label", menu_item_set_label);
+    glade_register_custom_prop (GTK_TYPE_MENU_ITEM, "use_underline", menu_item_set_use_underline);
+    glade_register_custom_prop (GTK_TYPE_MENU_ITEM, "use_stock", menu_item_set_use_stock);
 
     glade_register_widget (GTK_TYPE_ACCEL_LABEL, glade_standard_build_widget,
 			   NULL, NULL);
@@ -827,7 +992,7 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_NOTEBOOK, glade_standard_build_widget,
 			   notebook_build_children, NULL);
     glade_register_widget (GTK_TYPE_OPTION_MENU, glade_standard_build_widget,
-			   glade_standard_build_children, option_menu_find_internal_child);
+			   option_menu_build_children, option_menu_find_internal_child);
     glade_register_widget (GTK_TYPE_PIXMAP, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_PLUG, NULL,
