@@ -58,36 +58,12 @@ typedef enum {
     PARSER_WIDGET_CHILD_PACKING,
     PARSER_WIDGET_CHILD_PACKING_PROPERTY,
     PARSER_WIDGET_CHILD_AFTER_PACKING,
+    PARSER_WIDGET_CHILD_PLACEHOLDER,
+    PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER,
     PARSER_FINISH,
     PARSER_UNKNOWN
 } ParserState;
 
-#if 0
-static gchar *state_names[] = {
-    "START",
-    "GLADE_INTERFACE",
-    "REQUIRES",
-    "WIDGET",
-    "WIDGET_PROPERTY",
-    "WIDGET_ATK",
-    "WIDGET_ATK_PROPERTY",
-    "WIDGET_ATK_ACTION",
-    "WIDGET_ATK_RELATION",
-    "WIDGET_AFTER_ATK",
-    "WIDGET_SIGNAL",
-    "WIDGET_AFTER_SIGNAL",
-    "WIDGET_ACCEL",
-    "WIDGET_AFTER_ACCEL",
-    "WIDGET_CHILD",
-    "WIDGET_CHILD_AFTER_WIDGET",
-    "WIDGET_CHILD_PACKING",
-    "WIDGET_CHILD_PACKING_PROPERTY",
-    "WIDGET_CHILD_AFTER_PACKING",
-    "FINISH",
-    "UNKNOWN"
-};
-#endif
-
 typedef struct _GladeParseState GladeParseState;
 struct _GladeParseState {
     ParserState state;
@@ -303,6 +279,8 @@ handle_signal(GladeParseState *state, const xmlChar **attrs)
 	    info.after = attrs[i+1][0] == 'y';
 	else if (!strcmp(attrs[i], "object"))
 	    info.object = alloc_string(state->interface, attrs[i+1]);
+	else if (!strcmp(attrs[i], "last_modification_time"))
+	    /* Do nothing. */;
 	else
 	    g_warning("unknown attribute `%s' for <signal>.", attrs[i]);
     }
@@ -718,6 +696,10 @@ glade_parser_start_element(GladeParseState *state,
 	    state->accels = NULL;
 
 	    state->state = PARSER_WIDGET;
+	} else if (!strcmp(name, "placeholder")) {
+	    /* this isn't a real child, so knock off  the last ChildInfo */
+	    state->widget->n_children--;
+	    state->state = PARSER_WIDGET_CHILD_PLACEHOLDER;
 	} else {
 	    g_warning("Unexpected element <%s> inside <child>.", name);
 	    state->prev_state = state->state;
@@ -773,6 +755,18 @@ glade_parser_start_element(GladeParseState *state,
 	state->state = PARSER_UNKNOWN;
 	state->unknown_depth++;
 	break;
+    case PARSER_WIDGET_CHILD_PLACEHOLDER:
+	g_warning("<placeholder> should be empty.  Found <%s>.", name);
+	state->prev_state = state->state;
+	state->state = PARSER_UNKNOWN;
+	state->unknown_depth++;
+	break;
+    case PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER:
+	/* this is a placeholder <child> element -- ignore extra elements */
+	state->prev_state = state->state;
+	state->state = PARSER_UNKNOWN;
+	state->unknown_depth++;
+	break;
     case PARSER_FINISH:
 	g_warning("There should be no elements here.  Found <%s>.", name);
 	state->prev_state = state->state;
@@ -831,7 +825,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -853,7 +847,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -915,7 +909,7 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop) {
+	if (state->translate_prop && state->content->str[0] != '\0') {
 	    prop.value = alloc_string(state->interface,
 			dgettext(state->domain, state->content->str));
 	} else {
@@ -930,6 +924,16 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	    g_warning("should find </child> here.  Found </%s>", name);
 	state->state = PARSER_WIDGET_AFTER_ACCEL;
 	break;
+    case PARSER_WIDGET_CHILD_PLACEHOLDER:
+	if (strcmp(name, "placeholder") != 0)
+	    g_warning("should find </placeholder> here.  Found </%s>", name);
+	state->state = PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER;
+	break;
+    case PARSER_WIDGET_CHILD_AFTER_PLACEHOLDER:
+	if (strcmp(name, "child") != 0)
+	    g_warning("should find </child> here.  Found </%s>", name);
+	state->state = PARSER_WIDGET_AFTER_ACCEL;
+	break;
     case PARSER_FINISH:
 	g_warning("should not be closing any elements in this state");
 	break;
@@ -1095,11 +1099,15 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 
     if (xmlSAXUserParseFile(&glade_parser, &state, file) < 0) {
 	g_warning("document not well formed!");
-	glade_interface_destroy(state.interface);
+	if (state.interface)
+	    glade_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
 	g_warning("did not finish in PARSER_FINISH state!");
+	if (state.interface)
+	    glade_interface_destroy(state.interface);
+	return NULL;
     }
     return state.interface;
 }
@@ -1132,11 +1140,15 @@ glade_parser_parse_buffer(const gchar *buffer, gint len, const gchar *domain)
 
     if (xmlSAXUserParseMemory(&glade_parser, &state, buffer, len) < 0) {
 	g_warning("document not well formed!");
-	glade_interface_destroy(state.interface);
+	if (state.interface)
+	    glade_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
 	g_warning("did not finish in PARSER_FINISH state!");
+	if (state.interface)
+	    glade_interface_destroy(state.interface);
+	return NULL;
     }
     return state.interface;
 }
